@@ -126,10 +126,10 @@ mime_whitelist = (
 )
 
 # custom filename patterns
-english_patterns = "Artist Album Title Track Total Genre Date Year"
+english_patterns = "Artist Album Title Track Total Genre Date Year Timestamp"
 
 # traductors: These are the custom filename patterns. Only if it does make sense.
-locale_patterns = _("Artist Album Title Track Total Genre Date Year")
+locale_patterns = _("Artist Album Title Track Total Genre Date Year Timestamp")
 
 patterns_formats = (
 	"%(artist)s",
@@ -140,6 +140,7 @@ patterns_formats = (
 	"%(genre)s",
 	"%(date)s",
 	"%(year)s",
+	"%(timestamp)s",
 )
 
 # add english and locale
@@ -373,6 +374,10 @@ def filename_to_uri(filename):
 	Filename can be a relative or absolute path, or an uri.
 	"""
 	if vfs_exists(filename):
+		url = urlparse.urlparse(filename)
+		if not url.scheme:
+			filename = urllib.pathname2url(filename)
+
 		return str(gnomevfs.URI(filename))
 	else:
 		return "file://" + urllib.quote(os.path.abspath(filename))
@@ -515,6 +520,11 @@ class TargetNameGenerator:
 			if isinstance(dict[key], basestring):
 				dict[key] = dict[key].replace("/", "-")
 		
+		# add timestamp to substitution dict -- this could be split into more
+		# entries for more fine-grained control over the string by the user...
+		timestamp_string = time.strftime("%Y%m%d_%H_%M_%S")
+		dict["timestamp"] = timestamp_string
+		
 		pattern = os.path.join(self.subfolders, self.basename + self.suffix)
 		result = pattern % dict
 		if isinstance(result, unicode):
@@ -533,6 +543,7 @@ class TargetNameGenerator:
 			folder = root
 		else:
 			folder = self.folder
+			
 		result = os.path.join(folder, urllib.quote(result))
 
 		return result
@@ -1101,7 +1112,7 @@ class Converter(Decoder):
 
 	"""A background task for converting files to another format."""
 
-	def __init__(self, sound_file, output_filename, output_type, delete_original=False):
+	def __init__(self, sound_file, output_filename, output_type, delete_original=False, output_resample=False, resample_rate=48000):
 		#print "Converter()"
 		Decoder.__init__(self, sound_file)
 
@@ -1110,10 +1121,14 @@ class Converter(Decoder):
 		self.output_filename = output_filename
 		self.output_type = output_type
 		self.vorbis_quality = None
+		self.aac_quality = None
 		self.mp3_bitrate = None
 		self.mp3_mode = None
 		self.mp3_quality = None
 
+		self.output_resample = output_resample
+		self.resample_rate = resample_rate
+
 		self.overwrite = False
 		self.delete_original = delete_original
 
@@ -1133,6 +1148,13 @@ class Converter(Decoder):
 
 		self.add_command("audioconvert")
 		#TODO self.add_command("audioscale")
+
+		#Hacked in audio resampling support
+		if self.output_resample:
+			#print "Resampling to %dHz" % (self.resample_rate)
+			self.add_command("audioresample ! audio/x-raw-float,rate=%d" % 
+					 (self.resample_rate))
+			self.add_command("audioconvert")
 		
 		encoder = self.encoders[self.output_type]()
 		if not encoder:
@@ -1187,6 +1209,9 @@ class Converter(Decoder):
 	def set_vorbis_quality(self, quality):
 		self.vorbis_quality = quality
 
+	def set_aac_quality(self, quality):
+		self.aac_quality = quality
+
 	def set_mp3_mode(self, mode):
 		self.mp3_mode = mode
 
@@ -1194,7 +1219,8 @@ class Converter(Decoder):
 		self.mp3_quality = quality
 
 	def add_flac_encoder(self):
-		return "flacenc"
+		s = "flacenc mid-side-stereo=true quality=8"
+		return s
 
 	def add_wav_encoder(self):
 		return "wavenc"
@@ -1237,7 +1263,8 @@ class Converter(Decoder):
 		return cmd
 
 	def add_aac_encoder(self):
-		return "faac profile=2 ! ffmux_mp4"
+		return "faac profile=2 bitrate=%s ! ffmux_mp4" % \
+			(self.aac_quality * 1000)
 
 class FileList:
 	"""List of files added by the user."""
@@ -1470,7 +1497,11 @@ class PreferencesDialog:
 		"mp3-cbr-quality": 192,
 		"mp3-abr-quality": 192,
 		"mp3-vbr-quality": 3,
+		"aac-quality": 192,
 		"delete-original": 0,
+		"output-resample": 0,
+		"resample-rate": 48000,
+		"flac-speed": 0,
 	}
 
 	sensitive_names = ["vorbis_quality", "choose_folder", "create_subfolders",
@@ -1489,6 +1520,8 @@ class PreferencesDialog:
 		self.aprox_bitrate = glade.get_widget("aprox_bitrate")
 		self.quality_tabs = glade.get_widget("quality_tabs")
 		self.delete_original = glade.get_widget("delete_original")
+		self.resample_toggle = glade.get_widget("resample_toggle")
+		self.resample_rate = glade.get_widget("resample_rate")
 
 		self.target_bitrate = None
 		self.convert_setting_from_old_version()
@@ -1553,41 +1586,52 @@ class PreferencesDialog:
 
 		mime_type = self.get_string("output-mime-type")
 
-		widgets = {		 "audio/x-vorbis": (have_vorbisenc, "output_mime_type_ogg_vorbis"),
-						"audio/x-flac"	: (have_flacenc, "output_mime_type_flac"),
-						"audio/x-wav"		: (have_wavenc, "output_mime_type_wav"),
-						"audio/mpeg"		: (have_lame, "output_mime_type_mp3"),
-						"audio/x-m4a"       : (have_faac, "output_mime_type_aac"),
-					}
+		widgets = (	("audio/x-vorbis", have_vorbisenc),
+					("audio/mpeg"    , have_lame),
+					("audio/x-flac"  , have_flacenc),
+					("audio/x-wav"   , have_wavenc),
+					("audio/x-m4a"   , have_faac),
+					) # must be in same order in output_mime_type
 
 		# desactivate output if encoder plugin is not present
-		for mime, d in widgets.iteritems():
-			encoder, widget = d
-			if not encoder:
-				w = glade.get_widget(widget)
-				w.set_sensitive(False)
+		widget = glade.get_widget('output_mime_type')
+		model = widget.get_model()
+		self.present_mime_types = []
+		i = 0
+		for b in widgets:
+			mime, encoder_present = b
+			if not encoder_present:
+				del model[i]
 				if mime_type == mime:
 					mime_type = self.defaults["output-mime-type"]
-
+			else:
+				self.present_mime_types.append(mime)
+				i += 1
+		for i, mime in enumerate(self.present_mime_types):
+			if mime_type == mime:
+				widget.set_active(i)
+		self.change_mime_type(mime_type)
+		
 		# display information about mp3 encoding
 		if not have_lame:
 			w = glade.get_widget("lame_absent")
 			w.show()
-		
-		widget_name = widgets.get(mime_type, None)[1]
-
-		if widget_name:
-			w = glade.get_widget(widget_name)
-			w.set_active(True)
-			self.change_mime_type(mime_type)
 			
 		w = glade.get_widget("vorbis_quality")
 		quality = self.get_float("vorbis-quality")
-		quality_setting = {0:0 ,0.2:1 ,0.4:2 ,0.6:3 , 0.8:4}
+		quality_setting = {0:0 ,0.2:1 ,0.4:2 ,0.6:3 , 0.8:4, 1.0:5}
+		w.set_active(5)
 		for k, v in quality_setting.iteritems():
 			if abs(quality-k) < 0.01:
 				w.set_active(v)
 			
+		w = glade.get_widget("aac_quality")
+		quality = self.get_int("aac-quality")
+		quality_setting = {64:0, 96:1, 128:2, 192:3, 256:4, 320:5}
+		for k, v in quality_setting.iteritems():
+			if quality == k:
+				w.set_active(v)
+			
 		self.mp3_quality = glade.get_widget("mp3_quality")
 		self.mp3_mode = glade.get_widget("mp3_mode")
 
@@ -1601,17 +1645,23 @@ class PreferencesDialog:
 			iter = model.append()
 			model.set(iter, 0, desc)
 		w.set_active(self.get_int("name-pattern-index"))
-
 		
 		self.custom_filename.set_text(self.get_string("custom-filename-pattern"))
 		if self.basename_pattern.get_active() == len(self.basename_patterns)-1:
 			self.custom_filename_box.set_sensitive(True)
 		else:
 			self.custom_filename_box.set_sensitive(False)
-			
+
+		if self.get_int("output-resample"):
+			self.resample_toggle.set_active(self.get_int("output-resample"))
+			self.resample_rate.set_sensitive(1)
+			rr_entry = glade.get_widget("resample_rate-entry")
+			rr_entry.set_text("%d" % (self.get_int("resample-rate")))
+
 		self.update_example()
 
 	def update_selected_folder(self):
+		self.into_selected_folder.set_use_underline(False)
 		self.into_selected_folder.set_label(_("Into folder %s") % 
 			beautify_uri(self.get_string("selected-folder")))
 
@@ -1628,7 +1678,10 @@ class PreferencesDialog:
 			quality = int(quality)
 			bitrates = (64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 500)
 			bitrate = bitrates[quality]
-			
+		
+		elif mime_type == "audio/x-m4a":
+			bitrate = self.get_int("aac-quality")
+
 		elif mime_type == "audio/mpeg":
 			quality = {
 				"cbr": "mp3-cbr-quality",
@@ -1856,10 +1909,15 @@ class PreferencesDialog:
 						"audio/mpeg": 1,
 						"audio/x-flac": 2,
 						"audio/x-wav": 3,
-						"audio/x-m4a": 3,
+						"audio/x-m4a": 4,
 		}
 		self.quality_tabs.set_current_page(tabs[mime_type])
 
+	def on_output_mime_type_changed(self, combo):
+		self.change_mime_type(
+			self.present_mime_types[combo.get_active()]
+		)
+
 	def on_output_mime_type_ogg_vorbis_toggled(self, button):
 		if button.get_active():
 			self.change_mime_type("audio/x-vorbis")
@@ -1881,9 +1939,13 @@ class PreferencesDialog:
 			self.change_mime_type("audio/x-m4a")
 
 	def on_vorbis_quality_changed(self, combobox):
-		quality = (0,0.2,0.4,0.6,0.8)
+		quality = (0,0.2,0.4,0.6,0.8,1.0)
 		self.set_float("vorbis-quality", quality[combobox.get_active()])
-		
+		self.update_example()
+
+	def on_aac_quality_changed(self, combobox):
+		quality = (64, 96, 128, 192, 256, 320)
+		self.set_int("aac-quality", quality[combobox.get_active()])
 		self.update_example()
 
 	def change_mp3_mode(self, mode):
@@ -1899,14 +1961,13 @@ class PreferencesDialog:
 		quality = self.get_int(keys[mode])
 		
 		quality_to_preset = {
-			"cbr": {64:0, 96:1, 128:2, 192:3, 256:4},
-			"abr": {64:0, 96:1, 128:2, 192:3, 256:4},
-			"vbr": {9:0,	 7:1,		 5:2,		3:3,	1:4}, # inverted !
+			"cbr": {64:0, 96:1, 128:2, 192:3, 256:4, 320:5},
+			"abr": {64:0, 96:1, 128:2, 192:3, 256:4, 320:5},
+			"vbr": {9:0,   7:1,   5:2,   3:3,   1:4,   0:5}, # inverted !
 		}
-			
+		
 		if quality in quality_to_preset[mode]:
 			self.mp3_quality.set_active(quality_to_preset[mode][quality])
-		
 		self.update_example()
 
 	def on_mp3_mode_changed(self, combobox):
@@ -1921,15 +1982,23 @@ class PreferencesDialog:
 			"vbr": "mp3-vbr-quality"
 		}
 		quality = {
-			"cbr": (64, 96, 128, 192, 256),
-			"abr": (64, 96, 128, 192, 256),
-			"vbr": (9, 7, 5, 3, 1),
+			"cbr": (64, 96, 128, 192, 256, 320),
+			"abr": (64, 96, 128, 192, 256, 320),
+			"vbr": (9, 7, 5, 3, 1, 0),
 		}
 		mode = self.get_string("mp3-mode")
 		self.set_int(keys[mode], quality[mode][combobox.get_active()])
-
 		self.update_example()
 
+	def on_resample_rate_changed(self, combobox):
+		changeto = combobox.get_active_text()
+		if int(changeto) >= 2:
+			self.set_int("resample-rate", int(changeto))
+
+	def on_resample_toggle(self, rstoggle):
+		self.set_int("output-resample", rstoggle.get_active())
+		self.resample_rate.set_sensitive(rstoggle.get_active())
+
 
 class ConverterQueueCanceled(SoundConverterException):
 
@@ -2017,9 +2086,12 @@ class ConverterQueue(TaskQueue):
 				raise ConverterQueueCanceled()
 			
 		c = Converter(sound_file, output_filename, 
-						self.window.prefs.get_string("output-mime-type"),
-						self.window.prefs.get_int("delete-original"))
+			                        self.window.prefs.get_string("output-mime-type"),
+			                        self.window.prefs.get_int("delete-original"),
+			                        self.window.prefs.get_int("output-resample"),
+			                        self.window.prefs.get_int("resample-rate"))
 		c.set_vorbis_quality(self.window.prefs.get_float("vorbis-quality"))
+		c.set_aac_quality(self.window.prefs.get_int("aac-quality"))
 		
 		quality = {
 			"cbr": "mp3-cbr-quality",
@@ -2678,6 +2750,8 @@ def main():
 				tuple[2](optarg)
 				break
 
+	if args and not args[0]:
+		args = []
 	args = map(filename_to_uri, args)
 
 	jobs = int(get_option('jobs'))
